Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Windows.Forms
Imports System.ComponentModel.Design

Namespace MySmeter
	<DesignTimeVisible(True)> _
	<Category("Components")> _
	<Description("Smeter for radio receivers")> _
	Public Partial Class MySmeter
		Inherits UserControl

		'
		' --------------------------------------------- Smeter30
		'private float _zeroAngle = 0;         // angle in degree for the zero position
		'private float _dbAngle = 1.4f;        // angle in degree of 1 decibel span
		'private float _maxDecibels = 35;
		'private float _minDecibels = -35;
		' --------------------------------------------- Smeter40
		'private float _zeroAngle = -3;         // angle in degree for the zero position
		'private float _dbAngle = 1.28f;        // angle in degree of 1 decibel span
		'private float _maxDecibels = 41;
		'private float _minDecibels = -36;
		' --------------------------------------------- Smeter
		Private _zeroAngle As Single = 8
		' angle in degree for the zero position
		Private _dbAngle As Single = 1F
		' angle in degree of 1 decibel span
		Private _maxDecibels As Single = 41
		Private _minDecibels As Single = -57
		' --------------------------------------------- 
		Private _oldAngle As Double = -0.8
		Private _speed As Single = 0.1F
		Private _centerX As Single = 0.485F
		' rotation center X as a rapport of the width
		Private _centerY As Single = 1.6F
		' rotation center Y as a rapport of the height
		Private _lineLen As Single = 1.45F
		' line length as a rapport of the height
		Private _lineLenPixels As Single
		' line length in pixels
		' --------------------------------------------- 
		Private _armPen As New Pen(Color.FromArgb(90, 80, 60), 3)
		Private _transparentBrush As Brush = New SolidBrush(Color.FromArgb(150, 200, 200, 210))
		Private _textBrush As Brush = New SolidBrush(Color.FromArgb(30, 100, 0))

		Public _trimmingDecibels As Single
		Public _trimHi As Double
		Public _trimLow As Double
		Private g As Graphics
		Private _originalBackBitmap As Bitmap
		Private _backBitmap As Bitmap
		Private krad As Double = 2 * Math.PI / 360
		Private w As Integer, h As Integer
		Private x0 As Single, y0 As Single
		Private peakVeff As Double
		Private sw As New Stopwatch()

		Public Sub New()
			InitializeComponent()
			_originalBackBitmap = DirectCast(BackgroundImage.Clone(), Bitmap)
			sw.Reset()
			sw.Start()
		End Sub

		Private Sub MySmeter_Resize(sender As Object, e As EventArgs)
			w = ClientSize.Width
			h = ClientSize.Height
			_backBitmap = New Bitmap(w, h)
			g = Graphics.FromImage(_backBitmap)
			g.DrawImage(_originalBackBitmap, 0, 0, w, h)
			BackgroundImage = DirectCast(_backBitmap.Clone(), Bitmap)
			g = Graphics.FromImage(BackgroundImage)
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality
			x0 = w * _centerX
			y0 = h * _centerY
			_lineLenPixels = _lineLen * h
			Update(-0.8)
		End Sub

		Public WriteOnly Property TrimHi() As Double
			Set
				_trimHi = Math.Pow(10, value / 20)
			End Set
		End Property

		Public WriteOnly Property TrimLow() As Double
			Set
				_trimLow = value / 200 - 0.1
			End Set
		End Property

		Public Overloads Sub Update(decibels As Double)
			' ------------------------------------------------------ test
			'decibels = 0;
			'decibels = 30;
			'decibels = 40;
			'decibels = -99;
			' ------------------------------------------------------ Trim LOW and HI
			Dim veff As Double = Math.Pow(10, decibels / 20)
			veff += _trimLow
			veff *= _trimHi
			If veff < 0.0005 Then
				veff = 0.0005
			End If
			Dim db As Double = 20 * Math.Log10(veff)
			' ------------------------------------------------------ 
			If db < _minDecibels Then
				db = _minDecibels
			End If
			If db > _maxDecibels Then
				db = _maxDecibels
			End If
			Dim a As Double = krad * (_zeroAngle + _dbAngle * db)
			_oldAngle += (a - _oldAngle) * _speed
			Dim _sin As Single = CSng(Math.Sin(_oldAngle))
			Dim _cos As Single = CSng(Math.Cos(_oldAngle))
			' ------------------------------------------------------ draw
			g.Clear(BackColor)
			g.DrawImage(_backBitmap, 0, 0)
			g.DrawLine(_armPen, x0, y0, x0 + _sin * _lineLenPixels, y0 - _cos * _lineLenPixels)
			g.FillRectangle(_transparentBrush, 0, h - 12, w, 12)
			' ------------------------------------------------------ uV text
			veff *= 55
			If veff > peakVeff OrElse sw.ElapsedMilliseconds > 300 Then
				peakVeff = veff
				sw.Reset()
				sw.Start()
			End If
			If peakVeff >= 1000 Then
				g.DrawString((peakVeff * 0.001).ToString("0.0 mV"), Font, _textBrush, 2, 2)
			ElseIf peakVeff >= 10 Then
				g.DrawString(peakVeff.ToString("0 uV"), Font, _textBrush, 2, 2)
			Else
				g.DrawString(peakVeff.ToString("0.0 uV"), Font, _textBrush, 2, 2)
			End If
			'
			Invalidate()
		End Sub
	End Class
End Namespace
