Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Windows.Forms

Namespace NumericDisplay
	Friend NotInheritable Class FrequencyEditSeparator
		Inherits UserControl
		Implements IRenderable
		Private Const MaskedDigitTransparency As Single = 0.3F

		Private _image As Image
		Private _masked As Boolean
		Private _renderNeeded As Boolean
		Private ReadOnly _maskedAttributes As New ImageAttributes()

		Public Property Image() As Image
			Get
				Return _image
			End Get
			Set
				_image = value
			End Set
		End Property

		Public Property Masked() As Boolean
			Get
				Return _masked
			End Get
			Set
				If _masked <> value Then
					_masked = value
					_renderNeeded = True
				End If
			End Set
		End Property

		Public Sub New()
			DoubleBuffered = True
			UpdateStyles()

			Dim cm As New ColorMatrix()
			cm.Matrix33 = MaskedDigitTransparency
			_maskedAttributes.SetColorMatrix(cm, ColorMatrixFlag.[Default], ColorAdjustType.Bitmap)
		End Sub

		Public Sub Render() Implements IRenderable.Render
			If _renderNeeded Then
				Invalidate()
				_renderNeeded = False
			End If
		End Sub

		Protected Overrides Sub OnPaint(e As PaintEventArgs)
			If _image IsNot Nothing Then
				Dim attributes As ImageAttributes = If((_masked OrElse Not Parent.Enabled), _maskedAttributes, Nothing)
				e.Graphics.DrawImage(_image, New Rectangle(0, 0, Width, Height), 0F, 0F, _image.Width, _image.Height, _
					GraphicsUnit.Pixel, attributes)
			End If
		End Sub
	End Class
End Namespace
